{
  Source Expert 1.4

  Description:
    Allow open source files of specified types from specified folders.

  To install Source Expert:
    1. Build expert DLL with your own Delphi.
    2. Register expert (you can use SrcExp.reg to do it, but modify
       path to expert before).

  New in version 1.4:
    - Now you can customize view sequence of rooted folders by moving
      it up and down.
    - Some bugs fixed.
  New in version 1.3:
    - Now you can specify list of file types your need (but only 
      do it manually in "Masks" section in Registry).
  New in version 1.2:
    - Now DLL version.
    - Some bugs fixed.
  New in version 1.1:
    - Now you can specify list of folders with your favourite
      source files.
    - Searching files in subfolders only when need (once on
      expanding parent folder).

  Roman M. Mochalov
  E-mail: roman@infra.sar.nnov.ru
}

library SrcExp;

uses
  ShareMem, Windows, Classes, Forms, Menus, ExptIntf, ToolIntf, VirtIntf, 
  SrcInsp;

type

{ SourceExpert }

  TSourceExpert = class(TIExpert)
  private
    FExpertItem: TIMenuItemIntf;
    procedure ExpertItemClick(Sender: TIMenuItemIntf);
  public
    constructor Create;
    destructor Destroy; override;
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
  end;

{ SourceExpert }

constructor TSourceExpert.Create;
var
  MainMenu: TIMainMenuIntf;
  MenuItems, ViewObjInspItem, ViewMenu: TIMenuItemIntf;
begin
  inherited Create;
  MainMenu := ToolServices.GetMainMenu;
  if MainMenu <> nil then
  try
    MenuItems := MainMenu.GetMenuItems;
    if MenuItems <> nil then
    try
      ViewObjInspItem := MainMenu.FindMenuItem('ViewObjInspItem');
      if ViewObjInspItem <> nil then
      try
        ViewMenu := ViewObjInspItem.GetParent;
        if ViewMenu <> nil then
        try
          FExpertItem := ViewMenu.InsertItem(ViewObjInspItem.GetIndex + 1,
            '&Source Inspector', 'ViewSrcInspItem', '',
            ShortCut(VK_F11, [ssShift]), 0, 0, [mfVisible, mfEnabled],
            ExpertItemClick);
        finally
          ViewMenu.Free;
        end;
      finally
        ViewObjInspItem.Free;
      end;
    finally
      MenuItems.Free;
    end;
  finally;
    MainMenu.Free;
  end;
end;

destructor TSourceExpert.Destroy;
begin
  FExpertItem.Free;
  inherited Destroy;
end;

procedure TSourceExpert.ExpertItemClick(Sender: TIMenuItemIntf);
begin
  SourceInspector.Show;
  SourceInspector.SetFocus;
end;

function TSourceExpert.GetName: string;
begin
  Result := 'Source Expert v1.4 by Roman M. Mochalov';
end;

function TSourceExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TSourceExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TSourceExpert.GetIDString: string;
begin
  Result := 'Roman_M_Mochalov.Source_Expert'
end;

procedure DoneExpert; export;
begin
end;

function InitExpert(ToolServices: TIToolServices;
  RegisterProc: TExpertRegisterProc;
  var Terminate: TExpertTerminateProc): Boolean; export; stdcall;
begin
  Result := True;
  try
    ExptIntf.ToolServices := ToolServices;
    Application.Handle := ToolServices.GetParentHandle;
    RegisterProc(TSourceExpert.Create);
  except
    ToolServices.RaiseException(ReleaseException);
  end;
end;

exports
  InitExpert name ExpertEntryPoint resident;

begin
end.
