{
  Source Expert 1.1
  Folders Form

  Roman M. Mochalov
  E-mail: roman@infra.sar.nnov.ru
}

unit SrcFld;

interface

uses
  Windows, ShellAPI, ShlObj, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ComCtrls, Registry;

type

{ TFoldersForm }

  TFoldersForm = class(TForm)
    FoldersList: TListView;
    AddButton: TButton;
    RemoveButton: TButton;
    OKButton: TButton;
    CancelButton: TButton;
    FoldersImages: TImageList;
    procedure FormCreate(Sender: TObject);
    procedure FoldersListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FoldersListEditing(Sender: TObject; Item: TListItem;
      var AllowEdit: Boolean);
    procedure FoldersListEdited(Sender: TObject; Item: TListItem; var S: string);
    procedure FoldersListMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure AddButtonClick(Sender: TObject);
    procedure RemoveButtonClick(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
  private
    procedure AddFolder(const Caption, Folder: string);
  public
    function Execute: Boolean;
  end;

function EditFolders: Boolean;

implementation

{$R *.DFM}

function EditFolders: Boolean;
var
  FoldersForm: TFoldersForm;
begin
  FoldersForm := TFoldersForm.Create(nil);
  try
    Result := FoldersForm.Execute;
  finally
    FoldersForm.Free;
  end;
end;

{ TFoldersForm }

procedure TFoldersForm.AddFolder(const Caption, Folder: string);
const
  Flags = SHGFI_ICON or SHGFI_SMALLICON or SHGFI_SYSICONINDEX;
var
  Info: TSHFileInfo;
  Handle: THandle;
  Item: TListItem;
begin
  Item := FoldersList.Items.Add;
  Item.Caption := Caption;
  Item.SubItems.Add(Folder);
  Handle := SHGetFileInfo(PChar(Folder), 0, Info, SizeOf(Info), Flags);
  if Handle <> 0 then
  begin
    if FoldersImages.Handle <> Handle then
    begin
      FoldersImages.Handle := Handle;
      FoldersImages.ShareImages := True;
    end;
    Item.ImageIndex := Info.iIcon;
  end;
end;

function TFoldersForm.Execute: Boolean;
begin
  Result := ShowModal = mrOK;
end;

procedure TFoldersForm.FormCreate(Sender: TObject);
var
  I: Integer;
  Folders: TStringList;
begin
  Folders := TStringList.Create;
  try
    with TRegIniFile.Create('Software\Source Inspector\1.0') do
    try
      ReadSection('Folders', Folders);
      for I := 0 to Folders.Count - 1 do
        AddFolder(Folders[I], ReadString('Folders', Folders[I], ''));
    finally
      Free;
    end;
  finally
    Folders.Free;
  end;
end;

procedure TFoldersForm.FoldersListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  RemoveButton.Enabled := FoldersList.Selected <> nil;
end;

procedure TFoldersForm.FoldersListEditing(Sender: TObject; Item: TListItem;
  var AllowEdit: Boolean);
begin
  OKButton.Default := False;
end;

procedure TFoldersForm.FoldersListEdited(Sender: TObject; Item: TListItem;
  var S: string);
begin
  OKButton.Default := True;
end;

procedure TFoldersForm.FoldersListMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FoldersList.Selected := FoldersList.ItemFocused;
end;

function CallBack(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM): Integer; stdcall;
var
  X, Y: Integer;
  Rect: TRect;
begin
  Result := 0;
  if uMsg = BFFM_INITIALIZED then
  begin
    GetWindowRect(Wnd, Rect);
    X := (GetSystemMetrics(SM_CXSCREEN) - Rect.Right + Rect.Left) div 2;
    Y := (GetSystemMetrics(SM_CYSCREEN) - Rect.Bottom + Rect.Top) div 2;
    SetWindowPos(Wnd, 0, X, Y, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
    SendMessage(Wnd, WM_SETTEXT, 0, Longint(PChar('Source Folders')));
  end;
end;

procedure TFoldersForm.AddButtonClick(Sender: TObject);
var
  BrowseInfo: TBrowseInfo;
  ItemID: PItemIDList;
  Name, Path: PChar;
begin
  Name := StrAlloc(MAX_PATH);
  Path := StrAlloc(MAX_PATH);
  try
    FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
    with BrowseInfo do
    begin
      hwndOwner := Application.Handle;
      pidlRoot  := PItemIDList(CSIDL_DRIVES);
      pszDisplayName := Name;
      lpszTitle := 'Select folder with source files:';
      ulFlags := BIF_RETURNONLYFSDIRS;
      lpfn := CallBack;
    end;
    ItemID := SHBrowseForFolder(BrowseInfo);
    if ItemID <> nil then
    begin
      SHGetPathFromIDList(ItemID, Path);
      AddFolder(Path, Path);
    end;
  finally
    StrDispose(Path);
    StrDispose(Name);
  end;
end;

procedure TFoldersForm.RemoveButtonClick(Sender: TObject);
begin
  FoldersList.Selected.Free;
end;

procedure TFoldersForm.OKButtonClick(Sender: TObject);
var
  I: Integer;
begin
  with TRegIniFile.Create('Software\Source Inspector\1.0') do
  try
    EraseSection('Folders');
    with FoldersList do
      for I := 0 to Items.Count - 1 do
        WriteString('Folders', Items[I].Caption, Items[I].SubItems[0]);
  finally
    Free;
  end;
  ModalResult := mrOK;
end;

end.
