{
  Source Expert 1.31
  Expert Form

  Roman M. Mochalov
  E-mail: roman@infra.sar.nnov.ru
}

unit SrcInsp;

interface

uses
  Windows, ShellAPI, ShlObj, Messages, SysUtils, Classes, Controls, Forms,
  Menus, ComCtrls, CommCtrl, Registry;

type

{ TSourceInspector }

  TSourceInspector = class(TForm)
    StatusBar: TStatusBar;
    TreeView: TTreeView;
    ImageList: TImageList;
    PopupMenu: TPopupMenu;
    PopupOpenItem: TMenuItem;
    PopupExploreItem: TMenuItem;
    PopupN1Item: TMenuItem;
    PopupAddItem: TMenuItem;
    PopupDeleteItem: TMenuItem;
    PopupRenameItem: TMenuItem;
    PopupN2Item: TMenuItem;
    PopupMoveUpItem: TMenuItem;
    PopupMoveDownItem: TMenuItem;
    PopupN3Item: TMenuItem;
    PopupRefreshItem: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TreeViewDblClick(Sender: TObject);
    procedure TreeViewDeletion(Sender: TObject; Node: TTreeNode);
    procedure TreeViewEditing(Sender: TObject; Node: TTreeNode;
      var AllowEdit: Boolean);
    procedure TreeViewExpanding(Sender: TObject; Node: TTreeNode;
      var AllowExpansion: Boolean);
    procedure TreeViewKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure TreeViewMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PopupMenuPopup(Sender: TObject);
    procedure PopupOpenItemClick(Sender: TObject);
    procedure PopupExploreItemClick(Sender: TObject);
    procedure PopupAddItemClick(Sender: TObject);
    procedure PopupDeleteItemClick(Sender: TObject);
    procedure PopupRenameItemClick(Sender: TObject);
    procedure PopupMoveUpItemClick(Sender: TObject);
    procedure PopupMoveDownItemClick(Sender: TObject);
    procedure PopupRefreshItemClick(Sender: TObject);
  private
    FMasks: TStringList;
    function AddFolder(const Caption, Folder: string): TTreeNode;
    function AddNode(const Name: string; Node: TTreeNode): TTreeNode;
    function AddSubMark(Node: TTreeNode): TTreeNode;
    function DelSubMark(Node: TTreeNode): TTreeNode;
    function GetMasks: TStringList;
    function GetNodeFile(Node: TTreeNode): string;
    function GetNodeFolder(Node: TTreeNode): string;
    function GetNextFolderNode(Node: TTreeNode): TTreeNode;
    function GetPrevFolderNode(Node: TTreeNode): TTreeNode;
    procedure OpenNodeFile(Node: TTreeNode);
    procedure ScanNodeDir(Node: TTreeNode);
  protected
    procedure LoadFolders; virtual;
    procedure ReLoadFolders; virtual;
    procedure SaveFolders; virtual;
  public
    property Masks: TStringList read GetMasks;
  end;

function SourceInspector: TSourceInspector;

implementation

uses ExptIntf;

{$R *.DFM}

var
  FSourceInspector: TSourceInspector = nil;

function SourceInspector: TSourceInspector;
begin
  if FSourceInspector = nil then FSourceInspector := TSourceInspector.Create(nil);
  Result := FSourceInspector;
end;

{ Some routines }

function ExtractFolder(const Name: string): string;
var
  I: Integer;
begin
  I := Length(Name);
  while (I > 0) and (Name[I] = '\') do Dec(I);
  if I <> 0 then
    Result := Copy(Name, 1, I) else
    Result := '';
end;

function ExpandFolder(const Name: string): string;
begin
  Result := ExtractFolder(Name) + '\';
end;

function IsNameValid(const Name: string): Boolean;
const
  C: string = '*?<>|"';
var
  I, P: Integer;
begin
  P := Pos(':', Name);
  if (P <> 0) and (P <> 2) then
  begin
    Result := False;
    Exit;
  end;
  for I := 1 to Length(C) do
  begin
    P := Pos(C[I], Name);
    if P <> 0 then
    begin
      Result := False;
      Exit;
    end;
  end;
  if (Name = '.') or (Name = '..') then { ! only for this release }
  begin
    Result := False;
    Exit;
  end;
  Result := True;
end;

function IsFolder(const Name: string): Boolean;
var
  SR: TSearchRec;
begin
  Result := False;
  if FindFirst(ExtractFolder(Name), faDirectory, SR) = 0 then
  try
    Result := SR.Attr and faDirectory <> 0;
  finally
    FindClose(SR);
  end;
end;

function IsFolderEmpty(const Name: string; Masks: TStringList): Boolean;
var
  Info: TSHFileInfo;
  SR: TSearchRec;
  I: Integer;
begin
  SHGetFileInfo(PChar(Name), 0, Info, SizeOf(Info), SHGFI_ATTRIBUTES);
  if Info.dwAttributes and SFGAO_HASSUBFOLDER <> 0 then
  begin
    Result := False;
    Exit;
  end;
  for I := 0 to Masks.Count - 1 do
  begin
    if FindFirst(Name + Masks[I], faAnyFile and not faSysFile, SR) = 0 then
    try
      Result := False;
      Exit;
    finally
      FindClose(SR);
    end;
  end;
  Result := True;
end;

function IsFloppy(const Name: string): Boolean;
var
  Info: TSHFileInfo;
begin
  SHGetFileInfo(PChar(Name), 0, Info, SizeOf(Info), SHGFI_ATTRIBUTES);
  Result := Info.dwAttributes and SFGAO_REMOVABLE <> 0;
end;

{ TSourceInspector }

function TSourceInspector.AddFolder(const Caption, Folder: string): TTreeNode;
const
  Flags = SHGFI_SMALLICON or SHGFI_SYSICONINDEX;
var
  Info: TSHFileInfo;
  Handle: THandle;
begin
  Result := TreeView.Items.AddChild(nil, Caption);
  Result.Data := StrNew(PChar(Folder));
  Handle := SHGetFileInfo(PChar(Folder), 0, Info, SizeOf(Info), Flags);
  if Handle <> 0 then
  begin
    if ImageList.Handle <> Handle then
    begin
      ImageList.Handle := Handle;
      ImageList.ShareImages := True;
    end;
    Result.ImageIndex := Info.iIcon;
    Result.SelectedIndex := Info.iIcon;
  end;
end;

function TSourceInspector.AddNode(const Name: string; Node: TTreeNode): TTreeNode;
const
  Flags = SHGFI_DISPLAYNAME or SHGFI_SMALLICON or SHGFI_SYSICONINDEX;
var
  Info: TSHFileInfo;
  Handle: THandle;
begin
  Result := TreeView.Items.AddChild(Node, ExtractFileName(Name));
  Handle := SHGetFileInfo(PChar(Name), 0, Info, SizeOf(Info), Flags);
  if Handle <> 0 then
  begin
    if ImageList.Handle <> Handle then
    begin
      ImageList.Handle := Handle;
      ImageList.ShareImages := True;
    end;
    Result.Text := Info.szDisplayName;
    Result.ImageIndex := Info.iIcon;
    Result.SelectedIndex := Info.iIcon;
  end;
end;

function TSourceInspector.AddSubMark(Node: TTreeNode): TTreeNode;
var
  Folder: string;
begin
  Result := Node;
  Folder := GetNodeFolder(Node);
  if IsFloppy(Folder) or not IsFolderEmpty(Folder, Masks) then
    TreeView.Items.AddChild(Node, '"') else
    Result := nil;
end;

function TSourceInspector.DelSubMark(Node: TTreeNode): TTreeNode;
begin
  Result := Node;
  Node := Node.GetFirstChild;
  if (Node <> nil) and (Node.Text = '"') then
    TreeView.Items.Delete(Node) else
    Result := nil;
end;

function TSourceInspector.GetMasks: TStringList;
begin
  if FMasks = nil then FMasks := TStringList.Create;
  Result := FMasks;
end;

function TSourceInspector.GetNodeFile(Node: TTreeNode): string;
begin
  Result := '';
  if Node <> nil then
  begin
    if Node.Parent <> nil then
    begin
      Result := Node.Text;
      while Node.Parent.Parent <> nil do
      begin
        Node := Node.Parent;
        Result := Node.Text + '\' + Result;
      end;
      Result := ExtractFolder(PChar(Node.Parent.Data)) + '\' + Result;
    end else
      Result := ExtractFolder(PChar(Node.Data));
  end;
end;

function TSourceInspector.GetNodeFolder(Node: TTreeNode): string;
begin
  Result := '';
  if Node <> nil then
  begin
    if Node.Parent <> nil then
    begin
      Result := Node.Text + '\';
      while Node.Parent.Parent <> nil do
      begin
        Node := Node.Parent;
        Result := Node.Text + '\' + Result;
      end;
      Result := ExpandFolder(PChar(Node.Parent.Data)) + Result;
    end else
      Result := ExpandFolder(PChar(Node.Data));
  end;
end;

function TSourceInspector.GetNextFolderNode(Node: TTreeNode): TTreeNode;
var
  I: Integer;
begin
  if Node <> nil then
    with TreeView do
    begin
      I := Node.AbsoluteIndex + 1;
      while I < Items.Count - 1 do
      begin
        Result := Items[I];
        if Result.Data <> nil then Exit;
        Inc(I);
      end;
    end;
  Result := nil;
end;

function TSourceInspector.GetPrevFolderNode(Node: TTreeNode): TTreeNode;
var
  I: Integer;
begin
  if Node <> nil then
    with TreeView do
    begin
      I := Node.AbsoluteIndex - 1;
      while I >= 0 do
      begin
        Result := Items[I];
        if Result.Data <> nil then Exit;
        Dec(I);
      end;
    end;
  Result := nil;
end;

procedure TSourceInspector.OpenNodeFile(Node: TTreeNode);
var
  Folder: string;
  FileName: string;
  EditWnd: HWND;
begin
  Folder := GetNodeFolder(Node);
  if not IsFloppy(Folder) and not IsFolder(Folder) then
  begin
    FileName := GetNodeFile(Node);
    if FileExists(FileName) and (ToolServices <> nil) then
    begin
      ToolServices.OpenFile(FileName);
      EditWnd := FindWindow('TEditWindow', nil);
      if EditWnd <> 0 then
      begin
        if IsIconic(EditWnd) then ShowWindow(EditWnd, SW_RESTORE);
        SetForegroundWindow(EditWnd);
      end;
    end;
  end;
end;

procedure TSourceInspector.ScanNodeDir(Node: TTreeNode);
var
  SR: TSearchRec;
  I: Integer;
  Folder: string;
begin
  Screen.Cursor := crHourGlass;
  StatusBar.SimpleText := 'Scanning folder...';
  StatusBar.Update;
  try
    Folder := GetNodeFolder(Node);
    if FindFirst(Folder + '*.*', faDirectory and not faSysFile, SR) = 0 then
    try
      repeat
        if IsNameValid(SR.Name) then
        begin
          SR.Name := Folder + SR.Name;
          if IsFolder(SR.Name) then AddSubMark(AddNode(SR.Name, Node));
        end;
      until FindNext(SR) <> 0;
    finally
      FindClose(SR);
    end;
    for I := 0 to Masks.Count - 1 do
    begin
      if FindFirst(Folder + Masks[I], faAnyFile and not faSysFile, SR) = 0 then
      try
        repeat
          if IsNameValid(SR.Name) then
          begin
            SR.Name := Folder + SR.Name;
            if not IsFolder(SR.Name) then AddNode(SR.Name, Node);
          end;
        until FindNext(SR) <> 0;
      finally
        FindClose(SR);
      end;
    end;
  finally
    StatusBar.SimpleText := 'Ready';
    Screen.Cursor := crDefault;
  end;
end;

procedure TSourceInspector.LoadFolders;
var
  I: Integer;
  Folders: TStringList;
begin
  Screen.Cursor := crHourGlass;
  StatusBar.SimpleText := 'Loading folders...';
  StatusBar.Update;
  try
    TreeView.Items.Clear;
    Folders := TStringList.Create;
    try
      with TRegIniFile.Create('Software\Source Inspector\1.0') do
      try
        ReadSection('Masks', Masks);
        ReadSection('Folders', Folders);
        for I := 0 to Folders.Count - 1 do
          AddSubMark(AddFolder(Folders[I], ReadString('Folders', Folders[I], '')));
      finally
        Free;
      end;
    finally
      Folders.Free;
    end;
    TreeView.Update;
  finally
    StatusBar.SimpleText := 'Ready';
    Screen.Cursor := crDefault;
  end;
end;

procedure TSourceInspector.ReLoadFolders;
begin
  SaveFolders;
  LoadFolders;
end;

procedure TSourceInspector.SaveFolders;
var
  I: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    with TRegIniFile.Create('Software\Source Inspector\1.0') do
    try
      EraseSection('Folders');
      with TreeView do
        for I := 0 to Items.Count - 1 do
          if Items[I].Data <> nil then
            WriteString('Folders', Items[I].Text, PChar(Items[I].Data));
    finally
      Free;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TSourceInspector.FormCreate(Sender: TObject);
var
  SysMenu: HMENU;
begin
  SysMenu := GetSystemMenu(Handle, False);
  DeleteMenu(SysMenu, SC_RESTORE, MF_BYCOMMAND);
  DeleteMenu(SysMenu, SC_MINIMIZE, MF_BYCOMMAND);
  DeleteMenu(SysMenu, SC_MAXIMIZE, MF_BYCOMMAND);
  LoadFolders;
end;

procedure TSourceInspector.FormDestroy(Sender: TObject);
begin
  SaveFolders;
  FMasks.Free;
end;

procedure TSourceInspector.TreeViewDblClick(Sender: TObject);
begin
  OpenNodeFile(TreeView.Selected);
end;

procedure TSourceInspector.TreeViewDeletion(Sender: TObject; Node: TTreeNode);
begin
  if (Node <> nil) and (Node.Data <> nil) then
  begin
    StrDispose(PChar(Node.Data));
    Node.Data := nil;
  end;
end;

procedure TSourceInspector.TreeViewEditing(Sender: TObject; Node: TTreeNode;
  var AllowEdit: Boolean);
begin
  AllowEdit := Node.Data <> nil;
end;

procedure TSourceInspector.TreeViewExpanding(Sender: TObject; Node: TTreeNode;
  var AllowExpansion: Boolean);
begin
  if (Node <> nil) and (DelSubMark(Node) <> nil) then ScanNodeDir(Node);
end;

procedure TSourceInspector.TreeViewKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  with TreeView do
    if not IsEditing then
      case Key of
        VK_F5: ReLoadFolders;
        VK_F10: if ssShift in Shift then PopupMenu.Popup(ClientOrigin.X, ClientOrigin.Y);
        VK_RETURN: OpenNodeFile(Selected);
        VK_INSERT: PopupAddItemClick(Self);
        VK_DELETE: PopupDeleteItemClick(Self);
        VK_UP: if ssCtrl in Shift then PopupMoveUpItemClick(Self);
        VK_DOWN: if ssCtrl in Shift then PopupMoveDownItemClick(Self);
      end;
end;

procedure TSourceInspector.TreeViewMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Node: TTreeNode;
begin
  if Button = mbRight then
    with TreeView do
    begin
      Node := GetNodeAt(X, Y);
      Selected := Node;
    end;
end;

procedure TSourceInspector.PopupMenuPopup(Sender: TObject);
var
  Floppy: Boolean;
  Folder: Boolean;
begin
  with TreeView do
    if Selected <> nil then
    begin
      Floppy := IsFloppy(GetNodeFolder(Selected));
      if Floppy then
      begin
        PopupOpenItem.Enabled := False;
        PopupExploreItem.Enabled := True;
      end else
      begin
        Folder := IsFolder(GetNodeFolder(Selected));
        PopupOpenItem.Enabled := not Folder;
        PopupExploreItem.Enabled := Folder;
      end;
      PopupDeleteItem.Enabled := Selected.Data <> nil;
      PopupRenameItem.Enabled := Selected.Data <> nil;
      PopupMoveUpItem.Enabled := GetPrevFolderNode(Selected) <> nil;
      PopupMoveDownItem.Enabled := GetNextFolderNode(Selected) <> nil;
    end else
    begin
      PopupOpenItem.Enabled := False;
      PopupExploreItem.Enabled := False;
      PopupDeleteItem.Enabled := False;
      PopupRenameItem.Enabled := False;
      PopupMoveUpItem.Enabled := False;
      PopupMoveDownItem.Enabled := False;
    end;
end;

procedure TSourceInspector.PopupOpenItemClick(Sender: TObject);
begin
  OpenNodeFile(TreeView.Selected);
end;

procedure TSourceInspector.PopupExploreItemClick(Sender: TObject);
var
  FileName: string;
begin
  with TreeView do
    if Selected <> nil then
    begin
      FileName := GetNodeFolder(Selected);
      ShellExecute(Self.Handle, 'explore', PChar(FileName), nil, nil, SW_SHOW);
    end;
end;

function CallBack(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM): Integer; stdcall;
var
  X, Y: Integer;
  Rect: TRect;
begin
  Result := 0;
  if uMsg = BFFM_INITIALIZED then
  begin
    GetWindowRect(Wnd, Rect);
    X := (GetSystemMetrics(SM_CXSCREEN) - Rect.Right + Rect.Left) div 2;
    Y := (GetSystemMetrics(SM_CYSCREEN) - Rect.Bottom + Rect.Top) div 2;
    SetWindowPos(Wnd, 0, X, Y, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
    SendMessage(Wnd, WM_SETTEXT, 0, Longint(PChar('Source Inspector')));
  end;
end;

procedure TSourceInspector.PopupAddItemClick(Sender: TObject);
var
  BrowseInfo: TBrowseInfo;
  ItemID: PItemIDList;
  Name, Path: PChar;
begin
  Name := StrAlloc(MAX_PATH);
  Path := StrAlloc(MAX_PATH);
  try
    FillChar(BrowseInfo, SizeOf(BrowseInfo), 0);
    with BrowseInfo do
    begin
      hwndOwner := Application.Handle;
      pidlRoot  := PItemIDList(CSIDL_DRIVES);
      pszDisplayName := Name;
      lpszTitle := 'Select the folder with source files:';
      ulFlags := BIF_RETURNONLYFSDIRS;
      lpfn := CallBack;
    end;
    ItemID := SHBrowseForFolder(BrowseInfo);
    if ItemID <> nil then
    begin
      SHGetPathFromIDList(ItemID, Path);
      TreeView.Selected := AddSubMark(AddFolder(Path, Path));
      TreeView.Update;
    end;
  finally
    StrDispose(Path);
    StrDispose(Name);
  end;
end;

procedure TSourceInspector.PopupDeleteItemClick(Sender: TObject);

  function DeleteQuery(const Folder: string): Boolean;
  const
    Flags = MB_ICONQUESTION or MB_YESNO or MB_DEFBUTTON2;
  var
    Text: string;
  begin
    Text := Format('Delete item "%s" ?', [Folder]);
    Result := MessageBox(Handle, PChar(Text), PChar(Caption), Flags) = ID_YES;
  end;

begin
  with TreeView do
    if (Selected <> nil) and (Selected.Data <> nil) then
      if DeleteQuery(Selected.Text) then Selected.Delete;
end;

procedure TSourceInspector.PopupRenameItemClick(Sender: TObject);
begin
  with TreeView do
    if Selected <> nil then Selected.EditText;
end;

procedure TSourceInspector.PopupMoveUpItemClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  with TreeView do
    if (Selected <> nil) and (Selected.Data <> nil) then
    begin
      Node := GetPrevFolderNode(Selected);
      if Node <> nil then Selected.MoveTo(Node, naInsert);
    end;
end;

procedure TSourceInspector.PopupMoveDownItemClick(Sender: TObject);
var
  Node: TTreeNode;
begin
  with TreeView do
    if (Selected <> nil) and (Selected.Data <> nil) then
    begin
      Node := GetNextFolderNode(Selected);
      if Node <> nil then Node.MoveTo(Selected, naInsert);
    end;
end;

procedure TSourceInspector.PopupRefreshItemClick(Sender: TObject);
begin
  ReLoadFolders;
end;

initialization

finalization
  FSourceInspector.Free;
  FSourceInspector := nil;

end.
